@@ -5,14 +5,20 @@
   (C) 2005 by Gary Cramblitt <garycramblitt@comcast.net>
   -------------------
   Original author: Gary Cramblitt <garycramblitt@comcast.net>
- ******************************************************************************/
 
-/******************************************************************************
- *                                                                            *
- *    This program is free software; you can redistribute it and/or modify    *
- *    it under the terms of the GNU General Public License as published by    *
- *    the Free Software Foundation; version 2 of the License.                 *
- *                                                                            *
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 2 of the License, or
+  (at your option) any later version.
+
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  ******************************************************************************/
 
 // Qt includes.
@@ -115,7 +121,7 @@ void SbdThread::pushSsmlElem( SsmlElemType et, const QDomElement& elem )
     {
         case etSpeak: {
             SpeakElem e = m_speakStack.top();
-            for ( int ndx=0; ndx < attrCount; ndx++ )
+            for ( int ndx=0; ndx < attrCount; ++ndx )
             {
                 QDomAttr a = attrList.item( ndx ).toAttr();
                 if ( a.name() == "lang" )      e.lang = a.value();
@@ -131,7 +137,7 @@ void SbdThread::pushSsmlElem( SsmlElemType et, const QDomElement& elem )
             break; }
         case etProsody: {
             ProsodyElem e = m_prosodyStack.top();
-            for ( int ndx=0; ndx < attrCount; ndx++ )
+            for ( int ndx=0; ndx < attrCount; ++ndx )
             {
                 QDomAttr a = attrList.item( ndx ).toAttr();
                 if ( a.name() == "pitch" )    e.pitch = a.value();
@@ -145,7 +151,7 @@ void SbdThread::pushSsmlElem( SsmlElemType et, const QDomElement& elem )
             break; }
         case etEmphasis: {
             EmphasisElem e = m_emphasisStack.top();
-            for ( int ndx=0; ndx < attrCount; ndx++ )
+            for ( int ndx=0; ndx < attrCount; ++ndx )
             {
                 QDomAttr a = attrList.item( ndx ).toAttr();
                 if ( a.name() == "level" )    e.level = a.value();
@@ -154,7 +160,7 @@ void SbdThread::pushSsmlElem( SsmlElemType et, const QDomElement& elem )
             break; }
         case etPS: {
             PSElem e = m_psStack.top();
-            for ( int ndx=0; ndx < attrCount; ndx++ )
+            for ( int ndx=0; ndx < attrCount; ++ndx )
             {
                 QDomAttr a = attrList.item( ndx ).toAttr();
                 if ( a.name() == "lang" )     e.lang = a.value();
@@ -239,7 +245,7 @@ QString SbdThread::makeBreakElem( const QDomElement& e )
     QString s = "<break";
     QDomNamedNodeMap attrList = e.attributes();
     int attrCount = attrList.count();
-    for ( int ndx=0; ndx < attrCount; ndx++ )
+    for ( int ndx=0; ndx < attrCount; ++ndx )
     {
         QDomAttr a = attrList.item( ndx ).toAttr();
         s += makeAttr( a.name(), a.value() );
@@ -269,7 +275,11 @@ QString SbdThread::makeSentence( const QString& text )
     if ( !v.isEmpty() ) s += v;
     if ( !p.isEmpty() ) s += p;
     if ( !e.isEmpty() ) s += e;
-    s += text;
+    // Escape ampersands and less thans.
+    QString newText = text;
+    newText.replace(QRegExp("&(?!amp;)"), "&amp;");
+    newText.replace(QRegExp("<(?!lt;)"), "&lt;");
+    s += newText;
     if ( !e.isEmpty() ) s += "</emphasis>";
     if ( !p.isEmpty() ) s += "</prosody>";
     if ( !v.isEmpty() ) s += "</voice>";
@@ -345,7 +355,7 @@ QString SbdThread::parseSsmlNode( QDomNode& n, const QString& re )
             // kdDebug() << "SbdThread::parseSsmlNode: parsedPlainText = [" << d << "]" << endl;
             QStringList sentenceList = QStringList::split( '\t', s, false );
             int lastNdx = sentenceList.count() - 1;
-            for ( int ndx=0; ndx < lastNdx; ndx++ )
+            for ( int ndx=0; ndx < lastNdx; ++ndx )
             {
                 result += startSentence();
                 result += makeSentence( sentenceList[ndx] );
@@ -363,7 +373,7 @@ QString SbdThread::parseSsmlNode( QDomNode& n, const QString& re )
             QString s = parsePlainText( n.toCDATASection().data(), re );
             QStringList sentenceList = QStringList::split( '\t', s, false );
             int lastNdx = sentenceList.count() - 1;
-            for ( int ndx=0; ndx < lastNdx; ndx++ )
+            for ( int ndx=0; ndx < lastNdx; ++ndx )
             {
                 result += startSentence();
                 result += makeSentence( makeCDATA( sentenceList[ndx] ) );
@@ -542,9 +552,9 @@ bool SbdThread::event ( QEvent * e )
     {
         // kdDebug() << "SbdThread::event: emitting filteringFinished signal." << endl;
         emit filteringFinished();
-        return TRUE;
+        return true;
     }
-    else return FALSE;
+    else return false;
 }
 
 // ----------------------------------------------------------------------------
