@@ -1,19 +1,25 @@
 /***************************************************** vim:set ts=4 sw=4 sts=4:
   Configuration widget and functions for Festival (Interactive) plug in
   -------------------
-  Copyright : (C) 2004 Gary Cramblitt
+  Copyright:
+  (C) 2004 by Gary Cramblitt <garycramblitt@comcast.net>
   -------------------
   Original author: Gary Cramblitt <garycramblitt@comcast.net>
-  Current Maintainer: Gary Cramblitt <garycramblitt@comcast.net>
- ******************************************************************************/
 
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; version 2 of the License.               *
- *                                                                         *
- ***************************************************************************/
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 2 of the License, or
+  (at your option) any later version.
+
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ ******************************************************************************/
 
 // C++ includes.
 #include <math.h>
@@ -40,9 +46,9 @@
 #include <knuminput.h>
 #include <kprocio.h>
 #include <kprogress.h>
+#include <kiconloader.h>
 
 // KTTS includes.
-#include "pluginconf.h"
 #include "testplayer.h"
 
 // FestivalInt includes.
@@ -57,6 +63,7 @@ FestivalIntConf::FestivalIntConf( QWidget* parent, const char* name, const QStri
     // kdDebug() << "FestivalIntConf::FestivalIntConf: Running" << endl;
     m_festProc = 0;
     m_progressDlg = 0;
+    m_supportsSSML = FestivalIntProc::ssUnknown;
 
     QVBoxLayout *layout = new QVBoxLayout(this, KDialog::marginHint(),
         KDialog::spacingHint(), "FestivalIntConfigWidgetLayout");
@@ -119,10 +126,11 @@ FestivalIntConf::~FestivalIntConf(){
 * Given a voice code, returns index into m_voiceList array (and voiceCombo box).
 * -1 if not found.
 */
-int FestivalIntConf::voiceCodeToListIndex(const QString voiceCode)
+int FestivalIntConf::voiceCodeToListIndex(const QString& voiceCode) const
 {
-    for(uint index = 0 ; index < m_voiceList.count(); ++index){
-        // kdDebug() << "Testing: " << voiceSelected << " == " << m_voiceList[index].code << endl;
+    const int voiceListCount = m_voiceList.count();
+    for(int index = 0; index < voiceListCount; ++index){
+        // kdDebug() << "Testing: " << voiceCode << " == " << m_voiceList[index].code << endl;
         if(voiceCode == m_voiceList[index].code)
             return index;
     }
@@ -153,6 +161,8 @@ void FestivalIntConf::load(KConfig *config, const QString &configGroup){
     m_widget->preloadCheckBox->setChecked(config->readBoolEntry(
         "Preload", m_widget->preloadCheckBox->isChecked()));
     m_languageCode = config->readEntry("LanguageCode", m_languageCode);
+    m_supportsSSML = static_cast<FestivalIntProc::SupportsSSML>(
+        config->readNumEntry("SupportsSSML", FestivalIntProc::ssUnknown));
     QString codecName = PlugInProc::codecIndexToCodecName(
         m_widget->characterCodingBox->currentItem(), m_codecList);
     codecName = config->readEntry("Codec", codecName);
@@ -172,6 +182,7 @@ void FestivalIntConf::save(KConfig *config, const QString &configGroup){
     config->writeEntry("pitch", m_widget->frequencyBox->value());
     config->writeEntry("Preload", m_widget->preloadCheckBox->isChecked());
     config->writeEntry("LanguageCode", m_voiceList[m_widget->selectVoiceCombo->currentItem()].languageCode);
+    config->writeEntry("SupportsSSML", m_supportsSSML);
     int codec = m_widget->characterCodingBox->currentItem();
     config->writeEntry("Codec", PlugInProc::codecIndexToCodecName(codec, m_codecList));
 }
@@ -399,7 +410,7 @@ void FestivalIntConf::scanVoices()
     m_widget->selectVoiceCombo->setEnabled(false);
 
     // Clear existing list of supported voice codes.
-    m_supportedVoiceCodes.clear();
+    // m_supportedVoiceCodes.clear();
     m_widget->selectVoiceCombo->clear();
 
     QString exePath = realFilePath(m_widget->festivalPath->url());
@@ -438,6 +449,7 @@ void FestivalIntConf::scanVoices()
         if (!m_progressDlg->wasCancelled()) m_festProc->stopText();
         delete m_progressDlg;
         m_progressDlg = 0;
+        m_supportsSSML = m_festProc->supportsSSML();
     }
 
     if (!m_supportedVoiceCodes.isEmpty())
@@ -463,12 +475,14 @@ void FestivalIntConf::scanVoices()
 
         // Iterate thru list of voice codes returned by Festival,
         // find matching entry in voices.xml file, and add to list of supported voices.
+        QPixmap maleIcon = KGlobal::iconLoader()->loadIcon("male", KIcon::Small);
+        QPixmap femaleIcon = KGlobal::iconLoader()->loadIcon("female", KIcon::Small);
         QStringList::ConstIterator itEnd = m_supportedVoiceCodes.constEnd();
         for(QStringList::ConstIterator it = m_supportedVoiceCodes.begin(); it != itEnd; ++it )
         {
             QString code = *it;
             bool found = false;
-            for (uint index=0; index < voicesCount; index++)
+            for (uint index=0; index < voicesCount; ++index)
             {
                 QDomNode voiceNode = voices.item(index);
                 QString voiceCode = readXmlString(voiceNode, "code", QString::null);
@@ -488,7 +502,13 @@ void FestivalIntConf::scanVoices()
                     voiceTemp.rateAdjustable = readXmlBool(voiceNode, "rate-adjustable", true);
                     voiceTemp.pitchAdjustable = readXmlBool(voiceNode, "pitch-adjustable", true);
                     m_voiceList.append(voiceTemp);
-                    m_widget->selectVoiceCombo->insertItem(voiceTemp.name + " (" + voiceTemp.code + ")");
+                    QString voiceDisplayName = voiceTemp.name + " (" + voiceTemp.code + ")";
+                    if (voiceTemp.gender == "male")
+                        m_widget->selectVoiceCombo->insertItem(maleIcon, voiceDisplayName);
+                    else if (voiceTemp.gender == "female")
+                        m_widget->selectVoiceCombo->insertItem(femaleIcon, voiceDisplayName);
+                    else
+                        m_widget->selectVoiceCombo->insertItem(voiceDisplayName);
                     break;
                 }
             }
@@ -509,7 +529,7 @@ void FestivalIntConf::scanVoices()
             }
         }
         m_widget->selectVoiceCombo->setEnabled(true);
-    }
+    } else kdDebug() << "FestivalIntConf::scanVoices: No voices found" << endl;
     setDefaultVoice(voiceCodeToListIndex(currentVoiceCode));
     // Emit configChanged if the enabled state of the selectVoiceCombo has changed.
     // This occurs when user changes Festival EXE path, then clicks Rescan.
@@ -518,6 +538,7 @@ void FestivalIntConf::scanVoices()
 
 void FestivalIntConf::slotQueryVoicesFinished(const QStringList &voiceCodes)
 {
+    // kdDebug() << "FestivalIntConf::slotQueryVoicesFinished: voiceCodes.count() = " << voiceCodes.count() << endl;
     m_supportedVoiceCodes = voiceCodes;
     if (m_progressDlg) m_progressDlg->close();
 }
